@@ -5,7 +5,7 @@
 /*
  * Base64 decoding (ascii to binary).
  *
- * $Id: nssb64d.c,v 1.9 2012/04/25 14:50:16 gerv%gerv.net Exp $
+ * $Id: nssb64d.c,v 1.10 2012/11/27 22:48:09 bsmith%mozilla.com Exp $
  */
 
 #include "nssb64.h"
@@ -809,7 +809,7 @@ ATOB_AsciiToData(const char *string, unsigned int *lenp)
 ** Convert from ascii to binary encoding of an item.
 */
 SECStatus
-ATOB_ConvertAsciiToItem(SECItem *binary_item, char *ascii)
+ATOB_ConvertAsciiToItem(SECItem *binary_item, const char *ascii)
 {
     SECItem *dummy;
 
